import re


class LinePattern:
    """
    line_pattern is a list with at least two elements  - a string, which is a
    label and a regular expression compiled pattern (instance of
    re.Pattern) - the value to extract. String and regular expression can
    be in any order. The order matters for correct matching.

    If a documents has something like (price in Euro):

        EUR 10.23

    to extract 10.23 value - use:

        get_labeled_value(['EUR', re.compile('\d\d\.\d\d')])  # noqa

    If on the other hand, to extract value from:

        SUMME 10.23 EUR

    order is label value label, use:

        get_labeled_value(['SUMME', re.compile('\d\d\.\d\d'), 'EUR'])  # noqa

    line_pattern list may contain any number of strings - but just one
    compiled regular expression.
    """

    def __init__(self, line_pattern):
        self._line_pattern = line_pattern

    @property
    def line_pattern(self):
        return self._line_pattern

    def exact_match(self, line_of_words):
        if len(line_of_words) != len(self.line_pattern):
            return False

        counter = 0

        for index, value in enumerate(line_of_words):
            pattern = self.line_pattern[index]
            if isinstance(pattern, str):
                if pattern != value:
                    return False

            if isinstance(pattern, re.Pattern):
                if not re.match(pattern, value):
                    return False

            counter += 1

        return counter == len(self.line_pattern)

    def match(self, random_line_of_words):
        if len(random_line_of_words) < 2:
            return False

        if len(random_line_of_words) < len(self.line_pattern):
            return False

        for word_index, word in enumerate(random_line_of_words):
            segment = slice(word_index, word_index + len(self.line_pattern))
            matched = self.exact_match(
                list(random_line_of_words[segment])
            )
            if matched:
                return list(random_line_of_words[segment])

        return False

    def get_value(self, same_length_line_of_words):

        if len(same_length_line_of_words) != len(self.line_pattern):
            return False

        for index, value in enumerate(same_length_line_of_words):
            pattern = self.line_pattern[index]
            if isinstance(pattern, re.Pattern):
                matched = re.match(pattern, value)
                if matched:
                    return value

        return False



from lxml import html

from .line_pattern import LinePattern


class Hocron:
    """
    Receives hocr file type as input and provides
    a set of API function for metadata extraction.
    """

    def __init__(self, file_path):
        self.file_path = file_path
        self.doc = html.parse(file_path)

    @property
    def first_word(self):
        words = self.doc.xpath("//*[@class='ocrx_word']")

        if len(words) > 0:
            return words[0].text

        return None

    def get_labeled_value(self, line_pattern):
        """
        line_pattern is an instance of hocron.line_pattern.LinePattern
        """

        lines = self.doc.xpath("//*[@class='ocr_line']")

        for curr_line in lines:
            words = curr_line.xpath(".//*[@class='ocrx_word']")
            line_of_words = [w.text for w in words]
            matched_words = line_pattern.match(line_of_words)

            if matched_words:
                return line_pattern.get_value(matched_words)

        return None


__all__ = [
    'Hocron',
    'LinePattern'
]



import os
import re
import unittest

from hocron import Hocron, LinePattern

BASE_DIR = os.path.dirname(
    os.path.abspath(__file__)
)

DATA_DIR = os.path.join(
    BASE_DIR, "data"
)


def get_file_path(category, filename):
    return os.path.join(DATA_DIR, category, filename)


def get_hocr(category, filename):
    hocr = Hocron(
        get_file_path(category, filename)
    )
    return hocr


class TestBasic(unittest.TestCase):

    def test_first_word_lidl_1(self):

        hocr = get_hocr("lidl-receipts", "lidl-1.hocr")
        word = hocr.first_word

        self.assertTrue(word)
        self.assertRegex(word, 'L.D.$')

    def test_first_word_lidl_2(self):

        hocr = get_hocr("lidl-receipts", "lidl-2.hocr")
        word = hocr.first_word

        self.assertTrue(word)
        self.assertRegex(word, 'L.D.$')

    def test_first_word_lidl_3(self):

        hocr = get_hocr("lidl-receipts", "lidl-3.hocr")
        word = hocr.first_word

        self.assertTrue(word)
        self.assertRegex(word, 'L.D.$')

    def test_get_labeled_value_1(self):
        hocr = get_hocr("lidl-receipts", "lidl-1.hocr")

        line_pattern = LinePattern(
            ['EUR', re.compile('\d+[\.,]\d\d$')]
        )  # noqa
        value = hocr.get_labeled_value(line_pattern)

        self.assertTrue(value)
        self.assertEqual(value, '41,92')

    def test_get_labeled_value_2(self):
        hocr = get_hocr("lidl-receipts", "lidl-2.hocr")

        line_pattern = LinePattern(
            ['EUR', re.compile('\d+[\.,]\d\d$')]
        )  # noqa
        value = hocr.get_labeled_value(line_pattern)

        self.assertTrue(value)
        self.assertEqual(value, '49,36')

    def test_get_labeled_value_3(self):
        hocr = get_hocr("lidl-receipts", "lidl-3.hocr")

        line_pattern = LinePattern(
            ['EUR', re.compile('\d+[\.,]\d\d$')]
        )  # noqa
        value = hocr.get_labeled_value(line_pattern)

        self.assertTrue(value)
        self.assertEqual(value, '44,23')

    def test_get_labeled_value_for_date_1(self):
        hocr = get_hocr("lidl-receipts", "lidl-3.hocr")

        line_pattern = LinePattern(
            ['Datum', re.compile('\d\d.\d\d.\d\d')]
        )  # noqa
        value = hocr.get_labeled_value(line_pattern)

        self.assertTrue(value)
        self.assertEqual(value, '08.06,20')



import re
import unittest

from hocron import LinePattern


class TestLinePattern(unittest.TestCase):

    def test_line_pattern(self):
        line_pattern = LinePattern(['EUR', re.compile('\d\d\.\d\d')])  # noqa 
        matched_words = line_pattern.match(['EUR', '45.12'])

        self.assertTrue(matched_words)
        self.assertEquals(
            matched_words,
            ['EUR', '45.12']
        )

    def test_match(self):
        line_pattern = LinePattern(['Datum', re.compile('\d\d.\d\d.\d\d')])  # noqa
        exact_match = line_pattern.match(
            ['Datum', '08.06,20', '17:13', 'Uhr']
        )
        self.assertTrue(
            exact_match
        )

    def test_exact_match_1(self):
        line_pattern = LinePattern(['EUR', re.compile('\d\d\.\d\d')])  # noqa 

        self.assertTrue(
            line_pattern.exact_match(['EUR', '45.12'])
        )

        self.assertFalse(
            line_pattern.exact_match(['EUR', '45,12'])
        )

        self.assertFalse(
            line_pattern.exact_match(['45.12', 'EUR'])
        )

    def test_exact_match_2(self):
        line_pattern = LinePattern(['Datum', re.compile('\d\d.\d\d.\d\d')])  # noqa
        self.assertTrue(
            line_pattern.exact_match(
                ['Datum', '08.06,20']
            )
        )


if __name__ == '__main__':
    unittest.main()
